/* Class NutButtonWidget */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NutButtonWidget.h"
#include "Nut.h"
#include "options.h"
#include "db.h"
#include <stdlib.h>
#include <FL/Fl.H>

static NutButtonWidget *nbwobj[10];
static int nbwobjcount = 0;

//static void cb_exit(Fl_Widget *which)
//{
//exit(0);
//}

static void sync_all_nbws_cb (Fl_Widget *which, void *who)
{
NutButtonWidget *us_right_now = (NutButtonWidget *)who;
Fl_Widget* const *aptr = us_right_now->array();
int save = 0;
for (int i = 0; i <= MAX_SCREEN; i++) if (aptr[i]->visible()) for (int j = 0; j < nbwobjcount; j++) if (nbwobj[j] != us_right_now)
 {
 Fl_Widget* const *aptrthem = nbwobj[j]->array();
 nbwobj[j]->value(aptrthem[i]);
 save = i;
 }
if (save == MAX_SCREEN) exit(0);
//if (save == MAX_SCREEN) Fl::add_timeout(1.13, (Fl_Timeout_Handler)cb_exit, (void *)which);
if (save != MAX_SCREEN)
 {
 options.screen = save;
 write_OPTIONS();
 screen_change();
 }
}

NutButtonWidget::NutButtonWidget(int x, int y, int w, int h, Fl_Color widgetcolor, struct food *foodwork) : Fl_Tabs(x, y, w, h)
{
int i, j, k, tabheight = h / 22, packspacing = FL_NORMAL_SIZE / 3;
Fl_Pack *screen[MAX_SCREEN+1];
Fl_Pack *hpack[MAX_SCREEN], *column[MAX_SCREEN][6];
update_count = 0;
Fl_Group::current(0);
this->color(FL_WHITE,widgetcolor);
nbwobj[nbwobjcount++] = this;
this->callback(sync_all_nbws_cb,this);
const char *screentitle[] =
 {
 "Daily Value %","DV Amounts",
 "Carbs && Amino Acids","Miscellaneous",
 "Sat && Mono Fatty Acids","Poly && Trans Fatty Acids"
 };
for (j = 0; j < MAX_SCREEN; j++)
 {
 screen[j] = new Fl_Pack(x, y, w, h-tabheight, screentitle[j]);
 Fl_Group::current(0);
 wizard[j][0] = new Fl_Wizard(x, y, w, 6*h/22);
 wizard[j][0]->box(FL_NO_BOX);
 Fl_Group::current(0);
 hpack[j] = new Fl_Pack(x, y+6*h/22, w, 15*h/22);
 Fl_Group::current(0);
 screen[j]->add(wizard[j][0]);
 screen[j]->add(hpack[j]);
 hpack[j]->type(Fl_Pack::HORIZONTAL);
 screen[j]->labeltype(FL_EMBOSSED_LABEL);
 screen[j]->color(widgetcolor,widgetcolor);
 screen[j]->box(FL_NO_BOX);
 for (k = 0; k < 6; k++)
  {
  column[j][k] = new Fl_Pack(x+k*w/6, y, w/6, h-tabheight-packspacing);
  column[j][k]->box(FL_NO_BOX);
  column[j][k]->spacing(packspacing);
  Fl_Group::current(0);
  hpack[j]->add(column[j][k]);
  }
 this->add(screen[j]);
 }
screen[MAX_SCREEN] = new Fl_Pack(x, y, w, h-tabheight, "Quit NUT");
Fl_Group::current(0);
Fl_Box *has_ended = new Fl_Box(x, y, w, h-tabheight, "NUT has ended.");
has_ended->color(widgetcolor);
screen[MAX_SCREEN]->add(has_ended);
screen[MAX_SCREEN]->labeltype(FL_EMBOSSED_LABEL);
screen[MAX_SCREEN]->color(widgetcolor,widgetcolor);
this->add(screen[MAX_SCREEN]);
this->labelcolor(FL_YELLOW);

j = 0; k = 0; i = 1;
calorie_button = new NutButton(x, y, w/6, tabheight-packspacing, ENERC_KCAL, j, "Calories(2000)");
column[j][k]->add(calorie_button);
column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ENERC_KCAL, j, "Carb/Prot/Fat"));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 2; i < 14; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 0; k = 1; i = 1;
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j, 0));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 2; i < 14; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 0; k = 2;
for (i = 14; i < 16; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 18; i < 30; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 0; k = 3;
for (i = 14; i < 16; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 18; i < 30; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 0; k = 4;
for (i = 17; i > 15; i--) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 30; i < 40; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, FAPU, j, "Omega-6/3 Balance"));

j = 0; k = 5;
for (i = 17; i > 15; i--) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 30; i < 40; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, FAPU, j, 0, 0));

j = 1; k = 0; i = 1;
column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ENERC_KCAL, j, Nutrient[ENERC_KCAL]));
column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ENERC_KCAL, j, "Carb/Prot/Fat"));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 2; i < 14; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 1; k = 1; i = 1;
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j, 0));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 2; i < 14; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 1; k = 2;
for (i = 14; i < 16; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 18; i < 30; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 1; k = 3;
for (i = 14; i < 16; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 18; i < 30; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 1; k = 4;
for (i = 17; i > 15; i--) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 30; i < 40; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, FAPU, j, "Omega-6/3 Balance"));

j = 1; k = 5;
for (i = 17; i > 15; i--) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 30; i < 40; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));
column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, FAPU, j, 0, 0));

j = 2; k = 0;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 11; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 2; k = 1;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 11; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 2; k = 2;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 11; i < 22; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 2; k = 3;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 11; i < 22; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 2; k = 4;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 22; i < 32; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 2; k = 5;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 22; i < 32; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 3; k = 0;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 13; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 3; k = 1;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 13; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 3; k = 2;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 13; i < 25; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 3; k = 3;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 13; i < 25; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 3; k = 4;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 25; i < 36; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 3; k = 5;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 25; i < 36; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 4; k = 1;
for (i = 1; i < 16; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 4; k = 2;
for (i = 1; i < 16; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 4; k = 3;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 16; i < 28; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 4; k = 4;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 16; i < 28; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 5; k = 0;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 10; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 5; k = 1;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 1; i < 10; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 5; k = 2;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 10; i < 19; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 5; k = 3;
for (i = 0; i < 2; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 10; i < 19; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

j = 5; k = 4;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 19; i < 31; i++) column[j][k]->add(new NutButton(x, y, w/6, tabheight-packspacing, ScreenMap[j][i], j, Nutrient[ScreenMap[j][i]]));

j = 5; k = 5;
for (i = 0; i < 1; i++) column[j][k]->add(new Fl_Box(x, y, w/6, tabheight-packspacing));
for (i = 19; i < 31; i++) column[j][k]->add(new NutValue(x, y, w/6, tabheight-packspacing, widgetcolor, this, foodwork, ScreenMap[j][i], j));

this->value(screen[options.screen]);
}

void NutButtonWidget::add_me_to_the_update_list(NutValue *nv)
{
update_list[update_count++] = nv;
}

void NutButtonWidget::update(void)
{
for (int i = 0; i < update_count; i++) update_list[i]->update();
snprintf(calbuf,20,"Calories(%1.0f)",DV[ENERC_KCAL]);
calorie_button->label(calbuf);
}
