/* Class NutCharts */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NutCharts.h"
#include "meal.h"
#include "food.h"
#include "options.h"
#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Chart.H>
#include <string.h>

NutCharts::NutCharts (int x, int y, int w, int h, Fl_Color widgetcolor) : Fl_Pack (x, y, w, h)
{
this->color(widgetcolor);

new Fl_Box(x,y,w,2*FL_NORMAL_SIZE);
n = new Fl_Chart(x,y,w,h/4-5*FL_NORMAL_SIZE/2,"Nutrient");
new Fl_Box(x,y,w,2*FL_NORMAL_SIZE);
f = new Fl_Chart(x,y,w,h/4-5*FL_NORMAL_SIZE/2,"Fat Calories");
new Fl_Box(x,y,w,2*FL_NORMAL_SIZE);
c = new Fl_Chart(x,y,w,h/4-5*FL_NORMAL_SIZE/2,"Carb Calories");
new Fl_Box(x,y,w,2*FL_NORMAL_SIZE);
p = new Fl_Chart(x,y,w,h/4-5*FL_NORMAL_SIZE/2,"Protein Calories");
new Fl_Box(x,y,w,2*FL_NORMAL_SIZE);

n->type(FL_LINE_CHART);
f->type(FL_BAR_CHART);
c->type(FL_BAR_CHART);
p->type(FL_BAR_CHART);
n->box(FL_UP_BOX);
f->box(FL_UP_BOX);
c->box(FL_UP_BOX);
p->box(FL_UP_BOX);
n->color(widgetcolor);
f->color(widgetcolor);
c->color(widgetcolor);
p->color(widgetcolor);
n->autosize(1);
f->autosize(1);
c->autosize(1);
p->autosize(1);

this->end();
}

void NutCharts::populate(int nutnum, int screen)
{
int day = -1, i;
struct meal *meal_ptr = options.temp_meal_root;

n->clear();
f->clear();
c->clear();
p->clear();

for (i = 0; i < 1001; i++)
 {
 protein[i] = carb[i] = fat[i] = nut[i] = 0;
 mealdate[i][0] = '\0';
 }
Fl::check();
if (screen > 0 || DV[nutnum] == 0) n->label(Nutrient[nutnum]);
else
 {
 sprintf(labelbuf,"%s Daily Value %1.1f %s",Nutrient[nutnum],DV[nutnum],Unit[nutnum]);
 n->label(labelbuf);
 }

while (meal_ptr->next != NULL)
 {
 meal_ptr = meal_ptr->next;
 if (strcmp(mealdate[day],meal_ptr->meal_date) != 0) strcpy(mealdate[++day],meal_ptr->meal_date);
  protein[day] += FoodIndex[meal_ptr->food_no]->nutrient[PROT_KCAL] * meal_ptr->grams / 100;
  carb[day] += FoodIndex[meal_ptr->food_no]->nutrient[CHO_KCAL] * meal_ptr->grams / 100;
  fat[day] += FoodIndex[meal_ptr->food_no]->nutrient[FAT_KCAL] * meal_ptr->grams / 100;
  nut[day] += FoodIndex[meal_ptr->food_no]->nutrient[nutnum] * meal_ptr->grams / 100;
 if (day == 1000) break;
 }
Fl::check();
float maxn = 0, minn = 1e38, maxf = 0, maxc = 0, maxp = 0, max = 0;
float maxgraphpositions = 83 * FL_NORMAL_SIZE / 8;

if (options.defanal != 0) day = options.defanal / options.mealsperday;
for (i = day; i >= 0; i--)
 {
 if (nut[i] > maxn) maxn = nut[i];
 if (nut[i] < minn) minn = nut[i];
 if (fat[i] > maxf) maxf = fat[i];
 if (carb[i] > maxc) maxc = carb[i];
 if (protein[i] > maxp) maxp = protein[i];
 if (fat[i] + carb[i] + protein[i] > max) max = fat[i] + carb[i] + protein[i];
 }
if (screen == 0 && DV[nutnum] > 0)
 {
 maxn -= DV[nutnum];
 n->bounds(-DV[nutnum], DV[nutnum]);
 if (maxn > DV[nutnum]) n->bounds(-maxn, maxn);
 }
else
 {
 n->bounds(0, maxn);
 }

f->bounds(0, max);
c->bounds(0, max);
p->bounds(0, max);

if ( (int) maxgraphpositions > day)
 {
 if (screen == 0)
  {
  if (options.defanal < options.mealsperday)
   {
   n->add(nut[day] - DV[nutnum], 0, MEALCOLOR);
   f->add(fat[day], 0, MEALCOLOR);
   c->add(carb[day], 0, MEALCOLOR);
   p->add(protein[day], 0, MEALCOLOR);
   }
 
  for (i = day-1; i >= 0; i--)
   {
   n->add(nut[i] - DV[nutnum], 0, MEALCOLOR);
   f->add(fat[i], 0, MEALCOLOR);
   c->add(carb[i], 0, MEALCOLOR);
   p->add(protein[i], 0, MEALCOLOR);
   }
  }
 
 else
  {
  if (options.defanal < options.mealsperday)
   {
   n->add(nut[day], 0, MEALCOLOR);
   f->add(fat[day], 0, MEALCOLOR);
   c->add(carb[day], 0, MEALCOLOR);
   p->add(protein[day], 0, MEALCOLOR);
   }
 
  for (i = day-1; i >= 0; i--)
   {
   n->add(nut[i], 0, MEALCOLOR);
   f->add(fat[i], 0, MEALCOLOR);
   c->add(carb[i], 0, MEALCOLOR);
   p->add(protein[i], 0, MEALCOLOR);
   }
  }
 }

if ( (int) maxgraphpositions < day)
 {
 float factor = (float) day / maxgraphpositions;
 int index;
 if (screen == 0)
  {
  for (i = (int) maxgraphpositions-1; i >= 0; i--)
   {
   index = (int) (((float) i) * factor);
   n->add(nut[index] - DV[nutnum], 0, MEALCOLOR);
   f->add(fat[index], 0, MEALCOLOR);
   c->add(carb[index], 0, MEALCOLOR);
   p->add(protein[index], 0, MEALCOLOR);
   }
  }
 
 else
  {
  for (i = (int) maxgraphpositions-1; i >= 0; i--)
   {
   index = (int) (((float) i) * factor);
   n->add(nut[index], 0, MEALCOLOR);
   f->add(fat[index], 0, MEALCOLOR);
   c->add(carb[index], 0, MEALCOLOR);
   p->add(protein[index], 0, MEALCOLOR);
   }
  }
 }
}
